@@ -44,12 +44,10 @@ import os
 from StringIO import StringIO
 
 def number(v):
-    r = None
-    try:
-        r = int(v)
-    except ValueError:
-        r = float(v)
-    return r
+    if '.' in v:
+        return float(v)
+    else:
+        return int(v)
 
 class CljDecoder(object):
     def __init__(self, fd):
@@ -69,7 +67,7 @@ class CljDecoder(object):
         * type name
         * a flag to indicate if it's a collection
         """
-        if c.isdigit():
+        if c.isdigit() or c =='-':
             return ("number", False)
         elif c == 't' or c == 'f': ## true/false
             return ("boolean", False)
@@ -135,20 +133,25 @@ class CljDecoder(object):
                     v = False
 
             elif t == "char":
-                v = fd.read(1)
-                e = fd.read(1)
+                buf = []
+                while c is not self.terminator and c is not "" and c not in self.stop_chars:
+                    c = fd.read(1)
+                    buf.append(c)
+                
+                e = c
+                v = ''.join(buf[:-1])
 
             elif t == "nil":
                 e = fd.read(3)[-1]
                 v = None
 
             elif t == "number":
-                buf = [c]
+                buf = []
                 while c is not self.terminator and c is not "" and c not in self.stop_chars:
-                    c = fd.read(1)
                     buf.append(c)
+                    c = fd.read(1)
                 e = c
-                numstr = ''.join(buf[:-1])
+                numstr = ''.join(buf)
                 v = number(numstr)
 
             elif t == "keyword":
@@ -156,6 +159,7 @@ class CljDecoder(object):
                 while c is not self.terminator and c is not "" and c not in self.stop_chars:
                     c = fd.read(1)
                     buf.append(c)
+ 
                 e = c
                 v = ''.join(buf[:-1])
 
@@ -169,13 +173,14 @@ class CljDecoder(object):
                     c = fd.read(1)
                 e = c
                 #v = u''.join(buf).decode('unicode-escape')
-                v = u''.join(buf).encode("utf-8").decode('string-escape').decode('utf-8')
+                v = ''.join(buf).decode('string-escape')
             else:
                 r = False
                 e = c
 
             if e is self.terminator:
-                current_scope, self.terminator, self.container = self.value_stack.pop()
+                current_scope, _, self.container = self.value_stack.pop()
+
                 if r:
                     current_scope.append(v)
                     
@@ -190,6 +195,7 @@ class CljDecoder(object):
 
             if len(self.value_stack) > 0:
                 self.value_stack[-1][0].append(v)
+                self.terminator = self.value_stack[-1][1]
 
             return v
 
